//
//  ITable.m
//  ExamplePersistSQLite
//
//  Created by Fernando Nazario on 14/07/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "ITable.h"
#import "IDatabase.h"

@interface ITable(PrivateMethods)
- (void) insert;
- (void) update;
@end

static IDatabase *database = nil;
static NSMutableDictionary *tableCache = nil;

@implementation ITable

@synthesize primaryKey;
@synthesize idCustomer;
@synthesize savedInDatabase;


#pragma mark -
#pragma mark Set and Get Database

+ (void) setDatabase: (IDatabase *) newDatabase
{
	database = newDatabase;
}

+ (IDatabase *) database
{
	return database;
}

+ (void) assertDatabaseExists
{
	NSAssert1(database, @"Database not set.  Set the database using [ISModel setDatabase] before using ActiveRecord.", @"");
}

+ (NSString *) tableName
{
	return NSStringFromClass([self class]);
}

- (NSArray *) columns
{
	if(tableCache == nil)
	{
		tableCache = [NSMutableDictionary dictionary];
	}
	
	NSString *tableName = [[self class] tableName];
	NSArray *columns = [tableCache objectForKey:tableName];
	
	if(columns == nil)
	{
		columns = [database columnsForTableName:tableName];
		[tableCache setObject: columns forKey: tableName];
	}
	
	return columns;
}

- (NSArray *) columnsWithoutPrimaryKey
{
	NSMutableArray *columns = [NSMutableArray arrayWithArray: [self columns]];
	[columns removeObjectAtIndex:0];
	
	return columns;
}

- (NSArray *) propertyValues
{
	NSMutableArray *values = [NSMutableArray array];
	for(NSString *columnName in [self columnsWithoutPrimaryKey])
	{
		id value = [self valueForKey: columnName];
		
		if(value != nil)
			[values addObject: value];
		else
			[values addObject:[NSNull null]];
		
	}
	return values;
}

#pragma mark -
#pragma mark Insert/Update/Delete

- (void) beforeSave
{
	
}

- (void) save
{
	[[self class] assertDatabaseExists];
	
	[self beforeSave];
	
	if(!savedInDatabase)
		[self insert];
	else
		[self update];
}

- (void) insert
{
	NSMutableArray *parameterList = [NSMutableArray array];
	
	NSArray *columnsWithoutPrimaryKey = [self columnsWithoutPrimaryKey];
	
	for(int i = 0; i < [columnsWithoutPrimaryKey count]; i++)
	{
		[parameterList addObject: @"?"];
	}
	
	NSString *sql = [NSString stringWithFormat:@"insert into %@ (%@) values(%@)", [[self class] tableName], [columnsWithoutPrimaryKey componentsJoinedByString: @","], 
					 [parameterList componentsJoinedByString:@","]];
	
	[database executeSql:sql withParameters:[self propertyValues]];
	savedInDatabase = YES;
	primaryKey = [database lastInsertRowId];
}

- (void) update
{
	NSString *setValues = [[[self columnsWithoutPrimaryKey] componentsJoinedByString:@" = ?, "] stringByAppendingString:@" = ?"];
	NSString *sql = [NSString stringWithFormat:@"update %@ set %@ where primaryKey = ?", [[self class] tableName], setValues];
	
	NSArray *parameters = [[self propertyValues] arrayByAddingObject: [NSNumber numberWithUnsignedInt:primaryKey]];
	
	[database executeSql: sql withParameters: parameters];
	savedInDatabase = YES;
}

- (void) beforeDelete
{
	
}

- (void) delete
{
	[[self class] assertDatabaseExists];
	if(!savedInDatabase)
		return;
	
	[self beforeDelete];
	
	NSString *sql = [NSString stringWithFormat:@"delete from %@ where idCustomer = ?", [[self class] tableName]];
	//[database executeSqlWithParameters:sql, [NSNumber numberWithUnsignedInt:primaryKey], nil];

	[database executeSqlWithParameters:sql, [NSNumber numberWithUnsignedInt:idCustomer], nil];
	
	savedInDatabase = NO;
	primaryKey = 0;
}

#pragma mark -
#pragma mark Find Model Objects
+ (NSArray *) findWithSql: (NSString *)sql withParameters:(NSArray *)parameters
{
	[self assertDatabaseExists];
	
	NSArray *results = [database executeSql:sql withParameters: parameters withClassForRow: [self class]];
	
	[results setValue:[NSNumber numberWithBool:YES] forKey:@"savedInDatabase"];
	
	return results;
}

+ (NSArray *) findWithSql: (NSString *) sql
{
	return [self findWithSql:sql withParameters:nil];
}

+ (NSArray *) findAll
{
	return [self findWithSql: [NSString stringWithFormat:@"select * from %@", [self tableName]]];
}

@end
